@@ -35,6 +35,7 @@
 
 #include <string.h>
 #include <math.h>
+#include <stdlib.h>
 
 /**
  * SECTION:gcal-utils
@@ -1168,7 +1169,7 @@ is_source_enabled (ESource *source)
  */
 gboolean
 ask_recurrence_modification_type (GtkWidget      *parent,
-                                  ECalObjModType *modtype,
+                                  GcalRecurrenceModType *modtype,
                                   ESource        *source)
 {
   GtkWidget *dialog;
@@ -1178,7 +1179,7 @@ ask_recurrence_modification_type (GtkWidget      *parent,
   EClient *client;
 
   flags = GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT;
-  *modtype = E_CAL_OBJ_MOD_THIS;
+  *modtype = GCAL_RECURRENCE_MOD_THIS_ONLY;
 
   dialog = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (parent)),
                                    flags,
@@ -1210,15 +1211,15 @@ ask_recurrence_modification_type (GtkWidget      *parent,
         is_set = FALSE;
         break;
       case GTK_RESPONSE_ACCEPT:
-        *modtype = E_CAL_OBJ_MOD_THIS;
+        *modtype = GCAL_RECURRENCE_MOD_THIS_ONLY;
         is_set = TRUE;
         break;
       case GTK_RESPONSE_OK:
-        *modtype = E_CAL_OBJ_MOD_THIS_AND_FUTURE;
+        *modtype = GCAL_RECURRENCE_MOD_THIS_AND_FUTURE;
         is_set = TRUE;
         break;
       case GTK_RESPONSE_YES:
-        *modtype = E_CAL_OBJ_MOD_ALL;
+        *modtype = GCAL_RECURRENCE_MOD_ALL;
         is_set = TRUE;
         break;
       default:
@@ -1230,3 +1231,173 @@ ask_recurrence_modification_type (GtkWidget      *parent,
 
   return is_set;
 }
+
+struct
+{
+  const gchar        *territory;
+  GcalWeekDay         no_work_days;
+} no_work_day_per_locale[] = {
+  { "AE", GCAL_WEEK_DAY_FRIDAY   | GCAL_WEEK_DAY_SATURDAY } /* United Arab Emirates */,
+  { "AF", GCAL_WEEK_DAY_FRIDAY   | GCAL_WEEK_DAY_SATURDAY } /* Afghanistan */,
+  { "BD", GCAL_WEEK_DAY_FRIDAY   | GCAL_WEEK_DAY_SATURDAY } /* Bangladesh */,
+  { "BH", GCAL_WEEK_DAY_FRIDAY   | GCAL_WEEK_DAY_SATURDAY } /* Bahrain */,
+  { "BN", GCAL_WEEK_DAY_SUNDAY   | GCAL_WEEK_DAY_FRIDAY   } /* Brunei Darussalam */,
+  { "CR", GCAL_WEEK_DAY_SATURDAY                          } /* Costa Rica */,
+  { "DJ", GCAL_WEEK_DAY_FRIDAY                            } /* Djibouti */,
+  { "DZ", GCAL_WEEK_DAY_FRIDAY   | GCAL_WEEK_DAY_SATURDAY } /* Algeria */,
+  { "EG", GCAL_WEEK_DAY_FRIDAY   | GCAL_WEEK_DAY_SATURDAY } /* Egypt */,
+  { "GN", GCAL_WEEK_DAY_SATURDAY                          } /* Equatorial Guinea */,
+  { "HK", GCAL_WEEK_DAY_SATURDAY                          } /* Hong Kong */,
+  { "IL", GCAL_WEEK_DAY_FRIDAY   | GCAL_WEEK_DAY_SATURDAY } /* Israel */,
+  { "IQ", GCAL_WEEK_DAY_FRIDAY   | GCAL_WEEK_DAY_SATURDAY } /* Iraq */,
+  { "IR", GCAL_WEEK_DAY_THURSDAY | GCAL_WEEK_DAY_FRIDAY   } /* Iran */,
+  { "KW", GCAL_WEEK_DAY_FRIDAY   | GCAL_WEEK_DAY_SATURDAY } /* Kuwait */,
+  { "KZ", GCAL_WEEK_DAY_FRIDAY   | GCAL_WEEK_DAY_SATURDAY } /* Kazakhstan */,
+  { "LY", GCAL_WEEK_DAY_FRIDAY   | GCAL_WEEK_DAY_SATURDAY } /* Libya */,
+  { "MX", GCAL_WEEK_DAY_SATURDAY                          } /* Mexico */,
+  { "MY", GCAL_WEEK_DAY_FRIDAY   | GCAL_WEEK_DAY_SATURDAY } /* Malaysia */,
+  { "NP", GCAL_WEEK_DAY_SATURDAY                          } /* Nepal */,
+  { "OM", GCAL_WEEK_DAY_FRIDAY   | GCAL_WEEK_DAY_SATURDAY } /* Oman */,
+  { "QA", GCAL_WEEK_DAY_FRIDAY   | GCAL_WEEK_DAY_SATURDAY } /* Qatar */,
+  { "SA", GCAL_WEEK_DAY_FRIDAY   | GCAL_WEEK_DAY_SATURDAY } /* Saudi Arabia */,
+  { "SU", GCAL_WEEK_DAY_FRIDAY   | GCAL_WEEK_DAY_SATURDAY } /* Sudan */,
+  { "SY", GCAL_WEEK_DAY_FRIDAY   | GCAL_WEEK_DAY_SATURDAY } /* Syria */,
+  { "UG", GCAL_WEEK_DAY_SUNDAY                            } /* Uganda */,
+  { "YE", GCAL_WEEK_DAY_FRIDAY   | GCAL_WEEK_DAY_SATURDAY } /* Yemen */,
+};
+
+
+/**
+ * is_workday:
+ * @day: a guint representing the day of a week (0…Sunday, 6…Saturday)
+ *
+ * Checks whether @day is workday or not based on the Territory part of Locale.
+ *
+ * Returns: %TRUE if @day is a workday, %FALSE otherwise.
+ */
+gboolean
+is_workday (guint day)
+{
+  GcalWeekDay no_work_days;
+  gchar *locale;
+  gchar territory[3] = { 0, };
+  guint i;
+
+  if (day > 6)
+    return FALSE;
+
+  no_work_days = GCAL_WEEK_DAY_SATURDAY | GCAL_WEEK_DAY_SUNDAY;
+
+  locale = getenv ("LC_ALL");
+  if (!locale || g_utf8_strlen (locale, -1) < 5)
+    locale = getenv ("LC_TIME");
+
+  if (!locale)
+    {
+      g_warning ("Locale is NULL, assuming Saturday and Sunday as non workdays");
+      return !(no_work_days & 1 << day);
+    }
+
+  g_return_val_if_fail (g_utf8_strlen (locale, -1) >= 5, TRUE);
+
+  territory[0] = locale[3];
+  territory[1] = locale[4];
+
+  for (i = 0; i < G_N_ELEMENTS (no_work_day_per_locale); i++)
+    {
+      if (g_strcmp0 (territory, no_work_day_per_locale[i].territory) == 0)
+        {
+          no_work_days = no_work_day_per_locale[i].no_work_days;
+          break;
+        }
+    }
+
+  return !(no_work_days & 1 << day);
+}
+
+GList*
+filter_event_list_by_uid_and_modtype (GList                 *widgets,
+                                      GcalRecurrenceModType  mod,
+                                      const gchar           *uid)
+{
+  GcalEvent *event;
+  GList *result;
+  GList *l;
+
+  event = NULL;
+  result = NULL;
+
+  /* First pass: find the GcalEvent */
+  for (l = widgets; l != NULL; l = l->next)
+    {
+      GcalEventWidget *event_widget;
+      GcalEvent *ev;
+
+      event_widget = l->data;
+
+      /* Safeguard against stray widgets */
+      if (!GCAL_IS_EVENT_WIDGET (event_widget))
+        continue;
+
+      ev = gcal_event_widget_get_event (event_widget);
+
+      /*
+       * We can assume only one event will have the exact uuid. Even among
+       * recurrencies.
+       */
+      if (g_str_equal (uid, gcal_event_get_uid (ev)))
+        {
+          result = g_list_prepend (result, event_widget);
+          event = ev;
+        }
+    }
+
+  /* Second pass: find the other related events */
+  if (event && mod != GCAL_RECURRENCE_MOD_THIS_ONLY)
+    {
+      ECalComponentId *id;
+      ECalComponent *component;
+      ESource *source;
+      g_autofree gchar *id_prefix;
+
+      component = gcal_event_get_component (event);
+      source = gcal_event_get_source (event);
+      id = e_cal_component_get_id (component);
+      id_prefix = g_strdup_printf ("%s:%s", e_source_get_uid (source), id->uid);
+
+      for (l = widgets; l != NULL; l = l->next)
+        {
+          GcalEventWidget *event_widget;
+          GcalEvent *ev;
+
+          event_widget = l->data;
+
+          /* Safeguard against stray widgets */
+          if (!GCAL_IS_EVENT_WIDGET (event_widget))
+            continue;
+
+          ev = gcal_event_widget_get_event (event_widget);
+
+          if (g_str_equal (gcal_event_get_uid (ev), uid))
+            continue;
+
+          if (!g_str_has_prefix (gcal_event_get_uid (ev), id_prefix))
+            continue;
+
+          if (mod == GCAL_RECURRENCE_MOD_ALL)
+            {
+              result = g_list_prepend (result, event_widget);
+            }
+          else if (mod == GCAL_RECURRENCE_MOD_THIS_AND_FUTURE)
+            {
+              if (g_date_time_compare (gcal_event_get_date_start (event), gcal_event_get_date_start (ev)) < 0)
+                result = g_list_prepend (result, event_widget);
+            }
+
+        }
+
+      e_cal_component_free_id (id);
+    }
+
+  return result;
+}
