@@ -32,8 +32,7 @@ G_BEGIN_DECLS
 
 G_DECLARE_FINAL_TYPE (GcalWindow, gcal_window, GCAL, WINDOW, GtkApplicationWindow)
 
-GtkWidget*           gcal_window_new_with_view_and_date         (GcalApplication     *app,
-                                                                 GcalWindowViewType   view_type,
+GtkWidget*           gcal_window_new_with_date                  (GcalApplication     *app,
                                                                  icaltimetype        *date);
 
 void                 gcal_window_new_event                      (GcalWindow          *self);
