@@ -31,9 +31,6 @@ G_DECLARE_FINAL_TYPE (GcalWeekView, gcal_week_view, GCAL, WEEK_VIEW, GtkBox)
 
 GtkWidget*           gcal_week_view_new                          (void);
 
-void                 gcal_week_view_set_manager                  (GcalWeekView       *self,
-                                                                  GcalManager        *manager);
-
 void                 gcal_week_view_set_first_weekday            (GcalWeekView       *self,
                                                                   gint                day_nr);
 
